@@ -1,13 +1,13 @@
 /* This is just to make the code in the example directory link properly. */
 #include <libical/ical.h>
 
-int main()
+int main(void)
 {
-
     return 1;
 }
 
 
-void do_something(icalcomponent* comp){
+void do_something(icalcomponent* comp)
+{
     (void)comp;/*unused*/
 }
