@@ -51,6 +51,12 @@ icalattach *icalattach_new_from_url(const char *url)
     return attach;
 }
 
+static void attach_data_free(char *data, void *free_fn_data)
+{
+    _unused(free_fn_data);
+    free(data);
+}
+
 icalattach *icalattach_new_from_data(const char *data, icalattach_free_fn_t free_fn,
                                      void *free_fn_data)
 {
@@ -63,6 +69,16 @@ icalattach *icalattach_new_from_data(const char *data, icalattach_free_fn_t free
         return NULL;
     }
 
+    if (!free_fn) {
+        data = strdup(data);
+        if (!data) {
+            free(attach);
+            errno = ENOMEM;
+            return NULL;
+        }
+        free_fn = attach_data_free;
+    }
+
     attach->refcount = 1;
     attach->is_url = 0;
     attach->u.data.data = (char *) data;
