@@ -5,16 +5,16 @@
  *
  * (C) COPYRIGHT 2001, Critical Path
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
-  Or:
+ Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    The Mozilla Public License Version 2.0. You may obtain a copy of
+    the License at https://www.mozilla.org/MPL/
  */
 
 #include "icalproperty_cxx.h"
@@ -26,15 +26,14 @@ ICalProperty::ICalProperty() : imp(icalproperty_new(ICAL_ANY_PROPERTY))
 {
 }
 
-ICalProperty::ICalProperty(const ICalProperty &v) throw(icalerrorenum)
-  : imp(icalproperty_new_clone(v.imp))
+ICalProperty::ICalProperty(const ICalProperty &v) : imp(icalproperty_new_clone(v.imp))
 {
     if (imp == NULL) {
         throw icalerrno;
     }
 }
 
-ICalProperty &ICalProperty::operator=(const ICalProperty &v) throw(icalerrorenum)
+ICalProperty &ICalProperty::operator=(const ICalProperty &v)
 {
     if (this == &v) {
         return *this;
@@ -67,19 +66,19 @@ ICalProperty::ICalProperty(icalproperty *v) : imp(v)
 {
 }
 
-ICalProperty::ICalProperty(std::string str)
-  : imp(icalproperty_new_from_string(str.c_str()))
+ICalProperty::ICalProperty(std::string str) //NOLINT TODO:V3.1:pass const ref
+    : imp(icalproperty_new_from_string(str.c_str()))
 {
 }
 
 ICalProperty::ICalProperty(icalproperty_kind kind)
-  : imp(icalproperty_new(kind))
+    : imp(icalproperty_new(kind))
 {
 }
 
 std::string ICalProperty::as_ical_string()
 {
-    return (std::string)icalproperty_as_ical_string(imp);
+    return static_cast<std::string>(icalproperty_as_ical_string(imp));
 }
 
 icalproperty_kind ICalProperty::isa()
@@ -97,9 +96,11 @@ int ICalProperty::operator==(ICalProperty &rhs)
     icalparameter_xliccomparetype result;
     ICalValue  *thisPropValue = this->get_value();
     ICalValue  *rhsPropValue  = rhs.get_value();
-    result = icalvalue_compare((icalvalue *)*thisPropValue, (icalvalue *)*rhsPropValue);
+    result = icalvalue_compare(static_cast<icalvalue *>(*thisPropValue),
+                               static_cast<icalvalue *>(*rhsPropValue));
     delete thisPropValue;
-    return (result ==  ICAL_XLICCOMPARETYPE_EQUAL);
+    delete rhsPropValue;
+    return (result == ICAL_XLICCOMPARETYPE_EQUAL) ? 1 : 0;
 }
 
 void ICalProperty::add_parameter(ICalParameter &parameter)
@@ -119,12 +120,12 @@ void ICalProperty::set_parameter_from_string(const std::string &name, const std:
 
 std::string ICalProperty::get_parameter_as_string(const std::string &name)
 {
-    return (std::string)icalproperty_get_parameter_as_string(imp, name.c_str());
+    return static_cast<std::string>(icalproperty_get_parameter_as_string(imp, name.c_str()));
 }
 
-void ICalProperty::remove_parameter(const icalparameter_kind &kind)
+void ICalProperty::remove_parameter_by_kind(const icalparameter_kind &kind)
 {
-    icalproperty_remove_parameter(imp, kind);
+    icalproperty_remove_parameter_by_kind(imp, kind);
 }
 
 int ICalProperty::count_parameters()
@@ -148,7 +149,7 @@ ICalParameter *ICalProperty::get_next_parameter(const icalparameter_kind &kind)
 /** Access the value of the property */
 void ICalProperty::set_value(const ICalValue &val)
 {
-    icalproperty_set_value(imp, (ICalValue &)val);
+    icalproperty_set_value(imp, const_cast<ICalValue &>(val));
 }
 
 void ICalProperty::set_value_from_string(const std::string &val, const std::string &kind)
@@ -163,15 +164,15 @@ ICalValue *ICalProperty::get_value()
 
 std::string ICalProperty::get_value_as_string()
 {
-    return (std::string)icalproperty_get_value_as_string(imp);
+    return static_cast<std::string>(icalproperty_get_value_as_string(imp));
 }
 
-/** Return the name of the property -- the type name converted to a
+/** Returns the name of the property -- the type name converted to a
  *  string, or the value of get_x_name if the type is X property
  */
 std::string ICalProperty::get_name() const
 {
-    return (std::string)icalproperty_get_property_name(imp);
+    return static_cast<std::string>(icalproperty_get_property_name(imp));
 }
 
 /* Deal with X properties */
@@ -182,7 +183,7 @@ void ICalProperty::set_x_name(ICalProperty &prop, const std::string &name)
 
 std::string ICalProperty::get_x_name(ICalProperty &prop)
 {
-    return (std::string)icalproperty_get_x_name(prop);
+    return static_cast<std::string>(icalproperty_get_x_name(prop));
 }
 
 icalvalue_kind ICalProperty::value_to_value_kind(const icalparameter_value &val)
@@ -203,7 +204,7 @@ icalproperty_kind ICalProperty::value_kind_to_kind(const icalvalue_kind &kind)
 
 std::string ICalProperty::kind_to_string(const icalproperty_kind &kind)
 {
-    return (std::string)icalproperty_kind_to_string(kind);
+    return static_cast<std::string>(icalproperty_kind_to_string(kind));
 }
 
 icalproperty_kind ICalProperty::string_to_kind(const std::string &str)
@@ -213,7 +214,7 @@ icalproperty_kind ICalProperty::string_to_kind(const std::string &str)
 
 std::string ICalProperty::method_to_string(const icalproperty_method &method)
 {
-    return (std::string)icalproperty_method_to_string(method);
+    return static_cast<std::string>(icalproperty_method_to_string(method));
 }
 
 icalproperty_method ICalProperty::string_to_method(const std::string &str)
@@ -223,17 +224,17 @@ icalproperty_method ICalProperty::string_to_method(const std::string &str)
 
 std::string ICalProperty::enum_to_string(const int &e)
 {
-    return (std::string)icalproperty_enum_to_string(e);
+    return static_cast<std::string>(icalproperty_enum_to_string(e));
 }
 
-int ICalProperty::string_to_enum(const std::string &str)
+int ICalProperty::kind_and_string_to_enum(const icalproperty_kind &kind, const std::string &str)
 {
-    return icalproperty_string_to_enum(str.c_str());
+    return icalproperty_kind_and_string_to_enum(kind, str.c_str());
 }
 
-std::string ICalProperty::status_to_string(const icalproperty_status &s)
+std::string ICalProperty::status_to_string(const icalproperty_status &status)
 {
-    return (std::string)icalproperty_status_to_string(s);
+    return static_cast<std::string>(icalproperty_status_to_string(status));
 }
 
 icalproperty_status ICalProperty::string_to_status(const std::string &str)
@@ -276,7 +277,7 @@ void ICalProperty::set_attendee(const std::string &val)
 
 std::string ICalProperty::get_attendee() const
 {
-    return (std::string)icalproperty_get_attendee(imp);
+    return static_cast<std::string>(icalproperty_get_attendee(imp));
 }
 
 /* CALSCALE */
@@ -287,7 +288,7 @@ void ICalProperty::set_calscale(const std::string &val)
 
 std::string ICalProperty::get_calscale() const
 {
-    return (std::string)icalproperty_get_calscale(imp);
+    return static_cast<std::string>(icalproperty_get_calscale(imp));
 }
 
 /* CATEGORIES */
@@ -298,7 +299,7 @@ void ICalProperty::set_categories(const std::string &val)
 
 std::string ICalProperty::get_categories() const
 {
-    return (std::string)icalproperty_get_categories(imp);
+    return static_cast<std::string>(icalproperty_get_categories(imp));
 }
 
 /* CLASS */
@@ -309,7 +310,7 @@ void ICalProperty::set_class(const enum icalproperty_class &val)
 
 enum icalproperty_class ICalProperty::get_class() const
 {
-    return (enum icalproperty_class)icalproperty_get_class(imp);
+    return static_cast<enum icalproperty_class>(icalproperty_get_class(imp));
 }
 
 /* COMMENT */
@@ -320,7 +321,7 @@ void ICalProperty::set_comment(const std::string &val)
 
 std::string ICalProperty::get_comment() const
 {
-    return (std::string)icalproperty_get_comment(imp);
+    return static_cast<std::string>(icalproperty_get_comment(imp));
 }
 
 /* COMPLETED */
@@ -329,8 +330,7 @@ void ICalProperty::set_completed(const struct icaltimetype &val)
     icalproperty_set_completed(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_completed() const
-{
+struct icaltimetype ICalProperty::get_completed() const {
     return icalproperty_get_completed(imp);
 }
 
@@ -342,7 +342,7 @@ void ICalProperty::set_contact(const std::string &val)
 
 std::string ICalProperty::get_contact() const
 {
-    return (std::string)icalproperty_get_contact(imp);
+    return static_cast<std::string>(icalproperty_get_contact(imp));
 }
 
 /* CREATED */
@@ -351,8 +351,7 @@ void ICalProperty::set_created(const struct icaltimetype &val)
     icalproperty_set_created(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_created() const
-{
+struct icaltimetype ICalProperty::get_created() const {
     return icalproperty_get_created(imp);
 }
 
@@ -364,7 +363,7 @@ void ICalProperty::set_description(const std::string &val)
 
 std::string ICalProperty::get_description() const
 {
-    return (std::string)icalproperty_get_description(imp);
+    return static_cast<std::string>(icalproperty_get_description(imp));
 }
 
 /* DTEND */
@@ -373,8 +372,7 @@ void ICalProperty::set_dtend(const struct icaltimetype &val)
     icalproperty_set_dtend(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_dtend() const
-{
+struct icaltimetype ICalProperty::get_dtend() const {
     return icalproperty_get_dtend(imp);
 }
 
@@ -384,8 +382,7 @@ void ICalProperty::set_dtstamp(const struct icaltimetype &val)
     icalproperty_set_dtstamp(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_dtstamp() const
-{
+struct icaltimetype ICalProperty::get_dtstamp() const {
     return icalproperty_get_dtstamp(imp);
 }
 
@@ -395,8 +392,7 @@ void ICalProperty::set_dtstart(const struct icaltimetype &val)
     icalproperty_set_dtstart(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_dtstart() const
-{
+struct icaltimetype ICalProperty::get_dtstart() const {
     return icalproperty_get_dtstart(imp);
 }
 
@@ -406,8 +402,7 @@ void ICalProperty::set_due(const struct icaltimetype &val)
     icalproperty_set_due(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_due() const
-{
+struct icaltimetype ICalProperty::get_due() const {
     return icalproperty_get_due(imp);
 }
 
@@ -417,8 +412,7 @@ void ICalProperty::set_duration(const struct icaldurationtype &val)
     icalproperty_set_duration(imp, val);
 }
 
-struct icaldurationtype ICalProperty::get_duration() const
-{
+struct icaldurationtype ICalProperty::get_duration() const {
     return icalproperty_get_duration(imp);
 }
 
@@ -428,8 +422,7 @@ void ICalProperty::set_exdate(const struct icaltimetype &val)
     icalproperty_set_exdate(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_exdate() const
-{
+struct icaltimetype ICalProperty::get_exdate() const {
     return icalproperty_get_exdate(imp);
 }
 
@@ -450,8 +443,7 @@ void ICalProperty::set_exrule(const struct icalrecurrencetype &val)
     icalproperty_set_exrule(imp, val);
 }
 
-struct icalrecurrencetype ICalProperty::get_exrule() const
-{
+struct icalrecurrencetype ICalProperty::get_exrule() const {
     return icalproperty_get_exrule(imp);
 }
 
@@ -461,8 +453,7 @@ void ICalProperty::set_freebusy(const struct icalperiodtype &val)
     icalproperty_set_freebusy(imp, val);
 }
 
-struct icalperiodtype ICalProperty::get_freebusy() const
-{
+struct icalperiodtype ICalProperty::get_freebusy() const {
     return icalproperty_get_freebusy(imp);
 }
 
@@ -472,8 +463,7 @@ void ICalProperty::set_geo(const struct icalgeotype &val)
     icalproperty_set_geo(imp, val);
 }
 
-struct icalgeotype ICalProperty::get_geo() const
-{
+struct icalgeotype ICalProperty::get_geo() const {
     return icalproperty_get_geo(imp);
 }
 
@@ -483,8 +473,7 @@ void ICalProperty::set_lastmodified(const struct icaltimetype &val)
     icalproperty_set_lastmodified(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_lastmodified() const
-{
+struct icaltimetype ICalProperty::get_lastmodified() const {
     return icalproperty_get_lastmodified(imp);
 }
 
@@ -496,7 +485,7 @@ void ICalProperty::set_location(const std::string &val)
 
 std::string ICalProperty::get_location() const
 {
-    return (std::string)icalproperty_get_location(imp);
+    return static_cast<std::string>(icalproperty_get_location(imp));
 }
 
 /* MAXRESULTS */
@@ -540,7 +529,7 @@ void ICalProperty::set_organizer(const std::string &val)
 
 std::string ICalProperty::get_organizer() const
 {
-    return (std::string)icalproperty_get_organizer(imp);
+    return static_cast<std::string>(icalproperty_get_organizer(imp));
 }
 
 /* OWNER */
@@ -551,7 +540,7 @@ void ICalProperty::set_owner(const std::string &val)
 
 std::string ICalProperty::get_owner() const
 {
-    return (std::string)icalproperty_get_owner(imp);
+    return static_cast<std::string>(icalproperty_get_owner(imp));
 }
 
 /* PERCENT-COMPLETE */
@@ -584,7 +573,7 @@ void ICalProperty::set_prodid(const std::string &val)
 
 std::string ICalProperty::get_prodid() const
 {
-    return (std::string)icalproperty_get_prodid(imp);
+    return static_cast<std::string>(icalproperty_get_prodid(imp));
 }
 
 /* QUERY */
@@ -595,7 +584,7 @@ void ICalProperty::set_query(const std::string &val)
 
 std::string ICalProperty::get_query() const
 {
-    return (std::string)icalproperty_get_query(imp);
+    return static_cast<std::string>(icalproperty_get_query(imp));
 }
 
 /* QUERYNAME */
@@ -606,7 +595,7 @@ void ICalProperty::set_queryname(const std::string &val)
 
 std::string ICalProperty::get_queryname() const
 {
-    return (std::string)icalproperty_get_queryname(imp);
+    return static_cast<std::string>(icalproperty_get_queryname(imp));
 }
 
 /* RDATE */
@@ -615,8 +604,7 @@ void ICalProperty::set_rdate(const struct icaldatetimeperiodtype &val)
     icalproperty_set_rdate(imp, val);
 }
 
-struct icaldatetimeperiodtype ICalProperty::get_rdate() const
-{
+struct icaldatetimeperiodtype ICalProperty::get_rdate() const {
     return icalproperty_get_rdate(imp);
 }
 
@@ -626,8 +614,7 @@ void ICalProperty::set_recurrenceid(const struct icaltimetype &val)
     icalproperty_set_recurrenceid(imp, val);
 }
 
-struct icaltimetype ICalProperty::get_recurrenceid() const
-{
+struct icaltimetype ICalProperty::get_recurrenceid() const {
     return icalproperty_get_recurrenceid(imp);
 }
 
@@ -639,7 +626,7 @@ void ICalProperty::set_relatedto(const std::string &val)
 
 std::string ICalProperty::get_relatedto() const
 {
-    return (std::string)icalproperty_get_relatedto(imp);
+    return static_cast<std::string>(icalproperty_get_relatedto(imp));
 }
 
 /* RELCALID */
@@ -650,7 +637,7 @@ void ICalProperty::set_relcalid(const std::string &val)
 
 std::string ICalProperty::get_relcalid() const
 {
-    return (std::string)icalproperty_get_relcalid(imp);
+    return static_cast<std::string>(icalproperty_get_relcalid(imp));
 }
 
 /* REPEAT */
@@ -667,18 +654,15 @@ int ICalProperty::get_repeat() const
 /* REQUEST-STATUS */
 void ICalProperty::set_requeststatus(const std::string &val)
 {
-    icalreqstattype v;
-
-    v = icalreqstattype_from_string(val.c_str());
+    icalreqstattype v = icalreqstattype_from_string(val.c_str());
 
     icalproperty_set_requeststatus(imp, v);
 }
 
 std::string ICalProperty::get_requeststatus() const
 {
-    icalreqstattype v;
-    v = icalproperty_get_requeststatus(imp);
-    return (std::string)(icalreqstattype_as_string(v));
+    icalreqstattype v = icalproperty_get_requeststatus(imp);
+    return static_cast<std::string>(icalreqstattype_as_string(v));
 }
 
 /* RESOURCES */
@@ -689,7 +673,7 @@ void ICalProperty::set_resources(const std::string &val)
 
 std::string ICalProperty::get_resources() const
 {
-    return (std::string)icalproperty_get_resources(imp);
+    return static_cast<std::string>(icalproperty_get_resources(imp));
 }
 
 /* RRULE */
@@ -698,8 +682,7 @@ void ICalProperty::set_rrule(const struct icalrecurrencetype &val)
     icalproperty_set_rrule(imp, val);
 }
 
-struct icalrecurrencetype ICalProperty::get_rrule() const
-{
+struct icalrecurrencetype ICalProperty::get_rrule() const {
     return icalproperty_get_rrule(imp);
 }
 
@@ -711,7 +694,7 @@ void ICalProperty::set_scope(const std::string &val)
 
 std::string ICalProperty::get_scope() const
 {
-    return (std::string)icalproperty_get_scope(imp);
+    return static_cast<std::string>(icalproperty_get_scope(imp));
 }
 
 /* SEQUENCE */
@@ -744,7 +727,7 @@ void ICalProperty::set_summary(const std::string &val)
 
 std::string ICalProperty::get_summary() const
 {
-    return (std::string)icalproperty_get_summary(imp);
+    return static_cast<std::string>(icalproperty_get_summary(imp));
 }
 
 /* TARGET */
@@ -755,7 +738,7 @@ void ICalProperty::set_target(const std::string &val)
 
 std::string ICalProperty::get_target() const
 {
-    return (std::string)icalproperty_get_target(imp);
+    return static_cast<std::string>(icalproperty_get_target(imp));
 }
 
 /* TRANSP */
@@ -775,8 +758,7 @@ void ICalProperty::set_trigger(const struct icaltriggertype &val)
     icalproperty_set_trigger(imp, val);
 }
 
-struct icaltriggertype ICalProperty::get_trigger() const
-{
+struct icaltriggertype ICalProperty::get_trigger() const {
     return icalproperty_get_trigger(imp);
 }
 
@@ -788,7 +770,7 @@ void ICalProperty::set_tzid(const std::string &val)
 
 std::string ICalProperty::get_tzid() const
 {
-    return (std::string)icalproperty_get_tzid(imp);
+    return static_cast<std::string>(icalproperty_get_tzid(imp));
 }
 
 /* TZNAME */
@@ -799,7 +781,7 @@ void ICalProperty::set_tzname(const std::string &val)
 
 std::string ICalProperty::get_tzname() const
 {
-    return (std::string)icalproperty_get_tzname(imp);
+    return static_cast<std::string>(icalproperty_get_tzname(imp));
 }
 
 /* TZOFFSETFROM */
@@ -832,7 +814,7 @@ void ICalProperty::set_tzurl(const std::string &val)
 
 std::string ICalProperty::get_tzurl() const
 {
-    return (std::string)icalproperty_get_tzurl(imp);
+    return static_cast<std::string>(icalproperty_get_tzurl(imp));
 }
 
 /* UID */
@@ -843,7 +825,7 @@ void ICalProperty::set_uid(const std::string &val)
 
 std::string ICalProperty::get_uid() const
 {
-    return (std::string)icalproperty_get_uid(imp);
+    return static_cast<std::string>(icalproperty_get_uid(imp));
 }
 
 /* URL */
@@ -854,7 +836,7 @@ void ICalProperty::set_url(const std::string &val)
 
 std::string ICalProperty::get_url() const
 {
-    return (std::string)icalproperty_get_url(imp);
+    return static_cast<std::string>(icalproperty_get_url(imp));
 }
 
 /* VERSION */
@@ -865,7 +847,7 @@ void ICalProperty::set_version(const std::string &val)
 
 std::string ICalProperty::get_version() const
 {
-    return (std::string)icalproperty_get_version(imp);
+    return static_cast<std::string>(icalproperty_get_version(imp));
 }
 
 /* X */
@@ -876,7 +858,7 @@ void ICalProperty::set_x(const std::string &val)
 
 std::string ICalProperty::get_x() const
 {
-    return (std::string)icalproperty_get_x(imp);
+    return static_cast<std::string>(icalproperty_get_x(imp));
 }
 
 /* X-LIC-CLUSTERCOUNT */
@@ -887,7 +869,7 @@ void ICalProperty::set_xlicclustercount(const std::string &val)
 
 std::string ICalProperty::get_xlicclustercount() const
 {
-    return (std::string)icalproperty_get_xlicclustercount(imp);
+    return static_cast<std::string>(icalproperty_get_xlicclustercount(imp));
 }
 
 /* X-LIC-ERROR */
@@ -898,7 +880,7 @@ void ICalProperty::set_xlicerror(const std::string &val)
 
 std::string ICalProperty::get_xlicerror() const
 {
-    return (std::string)icalproperty_get_xlicerror(imp);
+    return static_cast<std::string>(icalproperty_get_xlicerror(imp));
 }
 
 /* X-LIC-MIMECHARSET */
@@ -909,7 +891,7 @@ void ICalProperty::set_xlicmimecharset(const std::string &val)
 
 std::string ICalProperty::get_xlicmimecharset() const
 {
-    return (std::string)icalproperty_get_xlicmimecharset(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimecharset(imp));
 }
 
 /* X-LIC-MIMECID */
@@ -920,7 +902,7 @@ void ICalProperty::set_xlicmimecid(const std::string &val)
 
 std::string ICalProperty::get_xlicmimecid() const
 {
-    return (std::string)icalproperty_get_xlicmimecid(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimecid(imp));
 }
 
 /* X-LIC-MIMECONTENTTYPE */
@@ -931,7 +913,7 @@ void ICalProperty::set_xlicmimecontenttype(const std::string &val)
 
 std::string ICalProperty::get_xlicmimecontenttype() const
 {
-    return (std::string)icalproperty_get_xlicmimecontenttype(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimecontenttype(imp));
 }
 
 /* X-LIC-MIMEENCODING */
@@ -942,7 +924,7 @@ void ICalProperty::set_xlicmimeencoding(const std::string &val)
 
 std::string ICalProperty::get_xlicmimeencoding() const
 {
-    return (std::string)icalproperty_get_xlicmimeencoding(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimeencoding(imp));
 }
 
 /* X-LIC-MIMEFILENAME */
@@ -953,7 +935,7 @@ void ICalProperty::set_xlicmimefilename(const std::string &val)
 
 std::string ICalProperty::get_xlicmimefilename() const
 {
-    return (std::string)icalproperty_get_xlicmimefilename(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimefilename(imp));
 }
 
 /* X-LIC-MIMEOPTINFO */
@@ -964,5 +946,5 @@ void ICalProperty::set_xlicmimeoptinfo(const std::string &val)
 
 std::string ICalProperty::get_xlicmimeoptinfo() const
 {
-    return (std::string)icalproperty_get_xlicmimeoptinfo(imp);
+    return static_cast<std::string>(icalproperty_get_xlicmimeoptinfo(imp));
 }
