@@ -121,28 +121,22 @@
 
 /* Copy the first part of user declarations.  */
 
-/* -*- Mode: C -*-                                                         */
 /*  ====================================================================== */
 /*  FILE: icalssyacc.y                                                     */
 /*  CREATOR: eric 08 Aug 2000                                              */
 /*                                                                         */
-/*  DESCRIPTION:                                                           */
-/*                                                                         */
-/*  $Id: icalssyacc.y,v 1.10 2008-01-14 00:35:26 dothebart Exp $           */
-/*  $Locker:  $                                                            */
-/*                                                                         */
-/*  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org        */
+/*  (C) COPYRIGHT 2000, Eric Busboom <eric@civicknowledge.com>             */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of either:                                           */
 /*                                                                         */
 /*    The LGPL as published by the Free Software Foundation, version       */
-/*    2.1, available at: http://www.fsf.org/copyleft/lesser.html           */
+/*    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html         */
 /*                                                                         */
 /*  Or:                                                                    */
 /*                                                                         */
-/*    The Mozilla Public License Version 1.0. You may obtain a copy of     */
-/*    the License at http://www.mozilla.org/MPL/                           */
+/*    The Mozilla Public License Version 2.0. You may obtain a copy of     */
+/*    the License at https://www.mozilla.org/MPL/                           */
 /*                                                                         */
 /* The Original Code is eric. The Initial Developer of the Original        */
 /* Code is Eric Busboom                                                    */
@@ -158,15 +152,15 @@
 
 extern struct icalgauge_impl *icalss_yy_gauge;
 
-#define YYPARSE_PARAM yy_globals
-#define YYLEX_PARAM yy_globals
+//#define YYPARSE_PARAM yy_globals
+//#define YYLEX_PARAM yy_globals
 #define YY_EXTRA_TYPE  icalgauge_impl*
 
 
-void sserror(char *s);
+void sserror(const char *s);
 
 static void ssyacc_add_where(struct icalgauge_impl* impl, char* prop,
-                        icalgaugecompare compare , char* value);
+                             icalgaugecompare compare , const char* value);
 static void ssyacc_add_select(struct icalgauge_impl* impl, char* str1);
 static void ssyacc_add_from(struct icalgauge_impl* impl, char* str1);
 static void set_logic(struct icalgauge_impl* impl,icalgaugelogic l);
@@ -346,7 +340,7 @@ YYID (i)
 #  ifndef YYSTACK_ALLOC_MAXIMUM
 #   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
 #  endif
-#  if (defined __cplusplus && ! defined _STDLIB_H \
+#  if (defined __cplusplus || defined __CYGWIN__ && ! defined _STDLIB_H \
        && ! ((defined YYMALLOC || defined malloc) \
              && (defined YYFREE || defined free)))
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
@@ -886,7 +880,7 @@ int yydebug;
 # define YYMAXDEPTH 10000
 #endif
 
-
+
 
 #if YYERROR_VERBOSE
 
@@ -1025,7 +1019,7 @@ yysyntax_error (char *yyresult, int yystate, int yychar)
 # endif
       char *yyfmt;
       char const *yyf;
-      static char const yyunexpected[] = "syntax error, unexpected %s";
+      static char const yyunexpected[] = "libical: syntax error, unexpected %s";
       static char const yyexpecting[] = ", expecting %s";
       static char const yyor[] = " or %s";
       char yyformat[sizeof yyunexpected
@@ -1097,7 +1091,7 @@ yysyntax_error (char *yyresult, int yystate, int yychar)
     }
 }
 #endif /* YYERROR_VERBOSE */
-
+
 
 /*-----------------------------------------------.
 | Release the memory associated to this symbol.  |
@@ -1129,7 +1123,7 @@ yydestruct (yymsg, yytype, yyvaluep)
         break;
     }
 }
-
+
 
 /* Prevent warnings from -Wmissing-prototypes.  */
 
@@ -1323,6 +1317,7 @@ yyparse ()
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
                   (unsigned long int) yystacksize));
 
+      /* coverity[OVERRUN] */
       if (yyss + yystacksize - 1 <= yyssp)
         YYABORT;
     }
@@ -1424,9 +1419,9 @@ yyreduce:
      users should not rely upon it.  Assigning to YYVAL
      unconditionally makes the parser a bit smaller, and it avoids a
      GCC warning that YYVAL may be used uninitialized.  */
+  /* coverity[uninit_use] */
   yyval = yyvsp[1-yylen];
 
-
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
@@ -1604,9 +1599,11 @@ yyerrorlab:
   /* Pacify compilers like GCC when the user code never invokes
      YYERROR and the label yyerrorlab therefore never appears in user
      code.  */
-  if (/*CONSTCOND*/ 0)
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunreachable-code"
+    if (/*CONSTCOND*/ 0)
      goto yyerrorlab;
-
+#pragma clang diagnostic pop
   /* Do not reclaim the symbols of the rule which action triggered
      this YYERROR.  */
   YYPOPSTACK (yylen);
@@ -1715,11 +1712,13 @@ yyreturn:
 
 
 static void ssyacc_add_where(struct icalgauge_impl* impl, char* str1,
-        icalgaugecompare compare , char* value_str)
+        icalgaugecompare compare , const char* value_str)
 {
 
     struct icalgauge_where *where;
-    char *compstr, *propstr, *c, *s,*l;
+    char *compstr, *propstr, *c, *l;
+    const char *s;
+    size_t lenstr;
 
     if ( (where = malloc(sizeof(struct icalgauge_where))) ==0){
         icalerror_set_errno(ICAL_NEWFAILED_ERROR);
@@ -1734,12 +1733,15 @@ static void ssyacc_add_where(struct icalgauge_impl* impl, char* str1,
 
     /* remove enclosing quotes */
     s = value_str;
-    if(*s == '\''){
-        s++;
-    }
-    l = s+strlen(s)-1;
-    if(*l == '\''){
-        *l=0;
+    lenstr = strlen(value_str);
+    if (lenstr > 1) {
+        if (*s == '\'') {
+            s++;
+        }
+        l = (char *)(&value_str[lenstr - 1]);
+        if (*l == '\'') {
+            *l=0;
+        }
     }
 
     where->value = strdup(s);
@@ -1757,7 +1759,7 @@ static void ssyacc_add_where(struct icalgauge_impl* impl, char* str1,
 
 
     /* Handle the case where a component was specified */
-    if(compstr != 0){
+    if (compstr != 0){
         where->comp = icalenum_string_to_component_kind(compstr);
     } else {
         where->comp = ICAL_NO_COMPONENT;
@@ -1851,12 +1853,12 @@ static void ssyacc_add_from(struct icalgauge_impl* impl, char* str1)
         assert(0);
     }
 
-    pvl_push(impl->from,(void*)ckind);
+    pvl_push(impl->from,(void *)ckind);
 
 }
 
 
-void sserror(char *s){
+void sserror(const char *s){
   fprintf(stderr,"Parse error \'%s\'\n", s);
   icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
 }
