@@ -2,23 +2,23 @@
  FILE: icalvcal.c
  CREATOR: eric 25 May 00
 
- (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
+ (C) COPYRIGHT 2000, Eric Busboom <eric@civicknowledge.com>
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
-    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
 
  Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
-    the License at http://www.mozilla.org/MPL/
+    The Mozilla Public License Version 2.0. You may obtain a copy of
+    the License at https://www.mozilla.org/MPL/
 
  The original code is icalvcal.c
 
  The icalvcal_convert routine calls icalvcal_traverse_objects to do
- its work.s his routine steps through through all of the properties
+ its work.s his routine steps through all of the properties
  and components of a VObject. For each name of a property or a
  component, icalvcal_traverse_objects looks up the name in
  conversion_table[]. This table indicates whether the name is of a
@@ -40,6 +40,7 @@
 #include "icalvcal.h"
 #include "icalerror.h"
 #include "icalvalue.h"
+#include "icaltimezone.h"
 #include "icalversion.h"        /* for ICAL_PACKAGE */
 
 #include <stddef.h>     /* for ptrdiff_t */
@@ -54,9 +55,9 @@ enum datatype
 };
 
 /* The indices must match between the strings and the codes. */
-static char *weekdays[] = { "SU", "MO", "TU", "WE", "TH", "FR", "SA" };
+static const char weekdays[][3] = { "SU", "MO", "TU", "WE", "TH", "FR", "SA" };
 
-static int weekday_codes[] = {
+static const int weekday_codes[] = {
     ICAL_SUNDAY_WEEKDAY,
     ICAL_MONDAY_WEEKDAY,
     ICAL_TUESDAY_WEEKDAY,
@@ -68,7 +69,7 @@ static int weekday_codes[] = {
 
 struct conversion_table_struct
 {
-    char *vcalname;
+    const char *vcalname;
     enum datatype type;
     void *(*conversion_func) (int icaltype, VObject *o, icalcomponent *comp,
                               icalvcal_defaults *defaults);
@@ -96,7 +97,7 @@ static icalproperty *create_parse_error_property(const char *message,
     return error_prop;
 }
 
-static char *get_string_value(VObject *object, int *free_string)
+static const char *get_string_value(VObject *object, int *free_string)
 {
     switch (vObjectValueType(object)) {
     case VCVT_USTRINGZ:
@@ -134,7 +135,10 @@ static void convert_floating_time_to_utc(struct icaltimetype *itt)
     t = mktime(&tmp_tm);
 
     /* Now convert back to a struct tm, but with a UTC time. */
-    gmtime_r(&t, &utc_tm);
+    if (!gmtime_r(&t, &utc_tm)) {
+        *itt = itt->is_date ? icaltime_null_date () : icaltime_null_time ();
+        return;
+    }
 
     /* Now put it back into the icaltime. */
     itt->year = utc_tm.tm_year + 1900;
@@ -144,8 +148,8 @@ static void convert_floating_time_to_utc(struct icaltimetype *itt)
     itt->minute = utc_tm.tm_min;
     itt->second = utc_tm.tm_sec;
 
-    /* Set the is_utc flag. */
-    itt->is_utc = 1;
+    /* Set the zone to UTC. */
+    itt->zone = icaltimezone_get_utc_timezone();
 }
 
 static void icalvcal_traverse_objects(VObject *,
@@ -283,7 +287,7 @@ static int get_alarm_properties(icalcomponent *comp, VObject *object,
     while (moreIteration(&iterator)) {
         VObject *eachProp = nextVObject(&iterator);
         const char *name = vObjectName(eachProp);
-        char *s;
+        const char *s;
         int free_string;
 
         s = get_string_value(eachProp, &free_string);
@@ -299,7 +303,7 @@ static int get_alarm_properties(icalcomponent *comp, VObject *object,
                 t.duration = icaldurationtype_null_duration();
 
                 /* If it is a floating time, convert it to a UTC time. */
-                if (!t.time.is_utc)
+                if (t.time.zone == NULL)
                     convert_floating_time_to_utc(&t.time);
 
                 /* Create a TRIGGER property. */
@@ -355,7 +359,7 @@ static int get_alarm_properties(icalcomponent *comp, VObject *object,
             }
 
         } else if (!strcmp(name, "TYPE")) {
-            char *fmttype = NULL;
+            const char *fmttype = NULL;
 
             if (!strcmp(s, "PCM")) {
                 fmttype = "audio/basic";
@@ -365,8 +369,9 @@ static int get_alarm_properties(icalcomponent *comp, VObject *object,
                 fmttype = "audio/x-wav";
             }
 
-            if (fmttype)
+            if (fmttype && !fmttype_param) {
                 fmttype_param = icalparameter_new_fmttype(fmttype);
+            }
 
             /* Display properties. */
         } else if (!strcmp(name, VCDisplayStringProp)) {
@@ -579,7 +584,7 @@ static void *transp_prop(int icaltype, VObject *object, icalcomponent *comp,
                          icalvcal_defaults *defaults)
 {
     icalproperty *prop = NULL;
-    char *s;
+    const char *s;
     int free_string;
 
     _unused(icaltype);
@@ -605,7 +610,7 @@ static void *sequence_prop(int icaltype, VObject *object, icalcomponent *comp,
                            icalvcal_defaults *defaults)
 {
     icalproperty *prop = NULL;
-    char *s;
+    const char *s;
     int free_string, sequence;
 
     _unused(icaltype);
@@ -637,7 +642,8 @@ static void *multivalued_prop(int icaltype, VObject *object, icalcomponent *comp
     icalproperty *prop = NULL;
     icalvalue *value;
     icalvalue_kind value_kind;
-    char *s, *tmp_copy, *p;
+    const char *s;
+    char *tmp_copy, *p;
     int free_string;
 
     _unused(comp);
@@ -673,7 +679,7 @@ static void *status_prop(int icaltype, VObject *object, icalcomponent *comp,
                          icalvcal_defaults *defaults)
 {
     icalproperty *prop = NULL;
-    char *s;
+    const char *s;
     int free_string;
     icalcomponent_kind kind;
 
@@ -727,7 +733,7 @@ static void *utc_datetime_prop(int icaltype, VObject *object, icalcomponent *com
     icalproperty_kind kind = (icalproperty_kind) icaltype;
     icalproperty *prop;
     icalvalue *value;
-    char *s;
+    const char *s;
     int free_string;
     struct icaltimetype itt;
 
@@ -742,7 +748,7 @@ static void *utc_datetime_prop(int icaltype, VObject *object, icalcomponent *com
     itt = icaltime_from_string(s);
 
     /* If it is a floating time, convert it to a UTC time. */
-    if (!itt.is_utc)
+    if (itt.zone == NULL)
         convert_floating_time_to_utc(&itt);
 
     value = icalvalue_new_datetime(itt);
@@ -757,7 +763,8 @@ static void *utc_datetime_prop(int icaltype, VObject *object, icalcomponent *com
 /* Parse the interval from the RRULE, returning a pointer to the first char
    after the interval and any whitespace. s points to the start of the
    interval. error_message is set if an error occurs. */
-static char *rrule_parse_interval(char *s, struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_interval(const char *s, struct icalrecurrencetype *recur,
+                                        const char **error_message)
 {
     int interval = 0;
 
@@ -788,7 +795,8 @@ static char *rrule_parse_interval(char *s, struct icalrecurrencetype *recur, cha
 /* Parse the duration from the RRULE, either a COUNT, e.g. '#5', or an UNTIL
    date, e.g. 20020124T000000. error_message is set if an error occurs.
    If no duration is given, '#2' is assumed. */
-static char *rrule_parse_duration(char *s, struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_duration(const char *s, struct icalrecurrencetype *recur,
+                                        const char **error_message)
 {
     /* If we've already found an error, just return. */
     if (*error_message)
@@ -811,7 +819,8 @@ static char *rrule_parse_duration(char *s, struct icalrecurrencetype *recur, cha
 
     } else if (*s >= '0' && *s <= '9') {
         /* If it starts with a digit it must be the UNTIL date. */
-        char *e, buffer[20];
+        const char *e;
+        char buffer[20];
         ptrdiff_t len;
 
         /* Find the end of the date. */
@@ -837,7 +846,7 @@ static char *rrule_parse_duration(char *s, struct icalrecurrencetype *recur, cha
            don't really know what timezone the vCalendar times are in. So if
            it can be converted to a DATE value, we do that. Otherwise we just
            use the current Unix timezone. Should be OK 99% of the time. */
-        if (!recur->until.is_utc) {
+        if (!icaltime_is_utc(recur->until)) {
             if (recur->until.hour == 0 &&
                 recur->until.minute == 0 &&
                 recur->until.second == 0) {
@@ -864,8 +873,9 @@ static char *rrule_parse_duration(char *s, struct icalrecurrencetype *recur, cha
     return s;
 }
 
-static char *rrule_parse_weekly_days(char *s,
-                                     struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_weekly_days(const char *s,
+                                           struct icalrecurrencetype *recur,
+                                           const char **error_message)
 {
     int i;
 
@@ -874,7 +884,7 @@ static char *rrule_parse_weekly_days(char *s,
         return NULL;
 
     for (i = 0; i < ICAL_BY_DAY_SIZE; i++) {
-        char *e = s;
+        const char *e = s;
         int found_day, day;
 
         found_day = -1;
@@ -892,7 +902,7 @@ static char *rrule_parse_weekly_days(char *s,
         if (found_day == -1)
             break;
 
-        /* cppcheck-suppress arrayIndexOutOfBounds since 'day' can't be >6 */
+        /* cppcheck-suppress arrayIndexOutOfBounds; since 'day' can't be >6 */
         recur->by_day[i] = weekday_codes[day];
 
         s = e;
@@ -908,8 +918,9 @@ static char *rrule_parse_weekly_days(char *s,
     return s;
 }
 
-static char *rrule_parse_monthly_days(char *s,
-                                      struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_monthly_days(const char *s,
+                                            struct icalrecurrencetype *recur,
+                                            const char **error_message)
 {
     int i;
 
@@ -918,14 +929,14 @@ static char *rrule_parse_monthly_days(char *s,
         return NULL;
 
     for (i = 0; i < ICAL_BY_MONTHDAY_SIZE; i++) {
-        char *e;
+        const char *e;
         int month_day;
 
         if (!strncmp(s, "LD", 2)) {
             month_day = -1;
             e = s + 2;
         } else {
-            month_day = strtol(s, &e, 10);
+            month_day = strtol(s, (char **)&e, 10);
 
             /* Check we got a valid day. */
             if (month_day < 1 || month_day > 31)
@@ -959,14 +970,15 @@ static char *rrule_parse_monthly_days(char *s,
     return s;
 }
 
-static char *rrule_parse_monthly_positions(char *s,
-                                           struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_monthly_positions(const char *s,
+                                                 struct icalrecurrencetype *recur,
+                                                 const char **error_message)
 {
     int occurrences[ICAL_BY_DAY_SIZE];
     int found_weekdays[7] = { 0, 0, 0, 0, 0, 0, 0 };
     int i, num_positions, elems, month_position, day;
     int num_weekdays, only_weekday = 0;
-    char *e;
+    const char *e;
 
     /* If we've already found an error, just return. */
     if (*error_message)
@@ -1005,7 +1017,7 @@ static char *rrule_parse_monthly_positions(char *s,
 
     /* Now read the weekdays in. */
     for (;;) {
-        char *e = s;
+        const char *e = s;
         int found_day, day;
 
         found_day = -1;
@@ -1075,8 +1087,9 @@ static char *rrule_parse_monthly_positions(char *s,
     return s;
 }
 
-static char *rrule_parse_yearly_months(char *s,
-                                       struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_yearly_months(const char *s,
+                                             struct icalrecurrencetype *recur,
+                                             const char **error_message)
 {
     int i;
 
@@ -1085,10 +1098,10 @@ static char *rrule_parse_yearly_months(char *s,
         return NULL;
 
     for (i = 0; i < ICAL_BY_MONTH_SIZE; i++) {
-        char *e;
+        const char *e;
         int month;
 
-        month = strtol(s, &e, 10);
+        month = strtol(s, (char **)&e, 10);
 
         /* Check we got a valid month. */
         if (month < 1 || month > 12)
@@ -1113,8 +1126,9 @@ static char *rrule_parse_yearly_months(char *s,
     return s;
 }
 
-static char *rrule_parse_yearly_days(char *s,
-                                     struct icalrecurrencetype *recur, char **error_message)
+static const char *rrule_parse_yearly_days(const char *s,
+                                           struct icalrecurrencetype *recur,
+                                           const char **error_message)
 {
     int i;
 
@@ -1170,7 +1184,7 @@ static void *rule_prop(int icaltype, VObject *object, icalcomponent *comp,
                        icalvcal_defaults *defaults)
 {
     icalproperty *prop = NULL;
-    char *s, *p, *parsestat, *error_message;
+    const char *s, *p, *parsestat, *error_message;
     const char *property_name;
     int free_string;
     struct icalrecurrencetype recur;
@@ -1244,7 +1258,7 @@ static void *rule_prop(int icaltype, VObject *object, icalcomponent *comp,
     return (void *)prop;
 }
 
-/* directly convertable property. The string representation of vcal is
+/* directly convertible property. The string representation of vcal is
    the same as ical */
 
 void *dc_prop(int icaltype, VObject *object, icalcomponent *comp, icalvcal_defaults *defaults)
@@ -1253,7 +1267,7 @@ void *dc_prop(int icaltype, VObject *object, icalcomponent *comp, icalvcal_defau
     icalproperty *prop;
     icalvalue *value;
     icalvalue_kind value_kind;
-    char *s;
+    const char *s;
 
 /*/,*t=0; */
     int free_string;
@@ -1467,7 +1481,7 @@ static void icalvcal_traverse_objects(VObject *object,
                                       icalproperty *last_prop, icalvcal_defaults *defaults)
 {
     VObjectIterator iterator;
-    char *name = "[No Name]";
+    const char *name;
     icalcomponent *subc = 0;
     int i;
 
@@ -1548,7 +1562,7 @@ static void icalvcal_traverse_objects(VObject *object,
                X_LIC_ERROR property to note this fact. */
 
             char temp[1024];
-            char *message = "Unsupported vCal property";
+            const char *message = "Unsupported vCal property";
             icalparameter *error_param;
             icalproperty *error_prop;
 
