@@ -9,12 +9,12 @@
 # it under the terms of either:
 #
 #   The LGPL as published by the Free Software Foundation, version
-#   2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.txt
+#   2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.txt
 #
 # Or:
 #
 #   The Mozilla Public License Version 2.0. You may obtain a copy of
-#   the License at http://www.mozilla.org/MPL/
+#   the License at https://www.mozilla.org/MPL/
 #
 ###############################################################################
 
@@ -33,7 +33,7 @@ assert(attach_url.get_is_url() == 1);
 retrieved_url = attach_url.get_url();
 assert(retrieved_url == dummy_url);
 
-attach_data = ICalGLib.Attach.new_from_data(dummy_data, ICalGLib.memory_free_buffer, None);
+attach_data = ICalGLib.Attach.new_from_data(dummy_data, None, None);
 assert(attach_data.get_is_url() == 0);
 retrieved_data = attach_data.get_data();
 assert(retrieved_data == dummy_data);
