@@ -1,15 +1,18 @@
 /*
 ======================================================================
 
-  The contents of this file are subject to the Mozilla Public License
-  Version 1.0 (the "License"); you may not use this file except in
-  compliance with the License. You may obtain a copy of the License at
-  http://www.mozilla.org/MPL/
+ (C) COPYRIGHT 2008, Eric Busboom <eric@civicknowledge.com>
 
-  Software distributed under the License is distributed on an "AS IS"
-  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-  the License for the specific language governing rights and
-  limitations under the License.
+ This library is free software; you can redistribute it and/or modify
+ it under the terms of either:
+
+    The LGPL as published by the Free Software Foundation, version
+    2.1, available at: https://www.gnu.org/licenses/lgpl-2.1.html
+
+ Or:
+
+    The Mozilla Public License Version 2.0. You may obtain a copy of
+    the License at https://www.mozilla.org/MPL/
 
 ======================================================================*/
 
@@ -21,11 +24,13 @@
 
 #include <stdlib.h>
 
-int main()
+#define MAX_FAILURE_RATE 1
+
+int main(void)
 {
     icalarray *timezones;
     icaltimezone *zone, *utc_zone;
-    char *zone_location;
+    const char *zone_location;
     size_t i;
     int ret = 0;
     unsigned int total_failed = 0;
@@ -55,7 +60,10 @@ int main()
     /* for all known time zones... */
     for (i = 0; i < timezones->num_elements; i++) {
         zone = (icaltimezone *)icalarray_element_at(timezones, i);
-        zone_location = (char *)icaltimezone_get_location(zone);
+        zone_location = icaltimezone_get_location(zone);
+        zonedef_printed = 0;
+        if (!zone_location)
+            continue;
 
         /*
          * select this location for glibc: needs support for TZ=<location>
@@ -66,7 +74,7 @@ int main()
 #else
         new_tz[0] = '\0';
         strncat(new_tz, "TZ=", 255);
-        strncat(new_tz, zone_location, 255);
+        strncat(new_tz, zone_location, 255 - strlen(new_tz));
         putenv(new_tz);
 #endif
         tzset();
@@ -75,7 +83,7 @@ int main()
          * the current zone and first day of first month in the year
          */
         start_time = time(NULL);
-        localtime_r(&start_time, &start_tm);
+        (void)localtime_r(&start_time, &start_tm);
         start_tm.tm_hour = 12;
         start_tm.tm_min = 0;
         start_tm.tm_sec = 0;
@@ -106,11 +114,13 @@ int main()
             if (verbose || curr_failed != failed) {
                 struct tm utc_tm;
 
-                gmtime_r(&curr_time, &utc_tm);
+                if (!gmtime_r(&curr_time, &utc_tm))
+                    memset(&utc_tm, 0, sizeof(utc_tm));
+
                 printf(
                     "%s: day %03d: %s: %04d-%02d-%02d %02d:%02d:%02d UTC = "
                     "libc %04d-%02d-%02d %02d:%02d:%02d dst %d",
-                    zone_location, day,
+                    icaltimezone_get_location(zone), day,
                     verbose ? (curr_failed ? "failed" : "okay") : (curr_failed ? "first failed" :
                                                                    "okay again"),
                     utc_tm.tm_year + 1900, utc_tm.tm_mon + 1, utc_tm.tm_mday, utc_tm.tm_hour,
@@ -131,7 +141,7 @@ int main()
                 printf("\n");
                 failed = curr_failed;
 
-                if (!zonedef_printed) {
+                if (failed && !zonedef_printed) {
                     icalcomponent *comp = icaltimezone_get_component(zone);
 
                     if (comp) {
@@ -153,14 +163,13 @@ int main()
         percent_failed = total_failed * 100 / (total_failed + total_okay);
         printf(" *** Summary: %lu zones tested, %u days failed, %u okay => %u%% failed ***\n",
                (unsigned long)timezones->num_elements, total_failed, total_okay, percent_failed);
-
-        if (!icaltzutil_get_exact_vtimezones_support()) {
-            if (!percent_failed) {
-                ret = 0;
-                printf(" *** Expect some small error rate with inter-operable vtimezones *** \n");
-            }
-       }
     }
 
+    if (percent_failed <= MAX_FAILURE_RATE && total_failed) {
+        ret = 0;
+        printf(" *** Expect some small error rate with inter-operable vtimezones *** \n");
+    }
+
+    icaltimezone_free_builtin_timezones();
     return ret;
 }
